#include "state.h"

#include "globals.h"
#include "utilities.h"
#include "state_registry.h"
#include "painting.h"

#include <algorithm>
#include <iostream>
#include <cassert>
using namespace std;


State::State(const PackedStateBin *buffer_, const StateRegistry &registry_,
             StateID id_)
    : buffer(buffer_),
      registry(&registry_),
      id(id_) {
    assert(buffer);
    assert(id != StateID::no_state);
}

State::~State() {
}

int State::operator[](size_t index) const {
    assert(!g_use_redblack_search || g_painting->is_black_var(index));
    return g_state_packer->get(buffer, index);
}

bool State::has_fact(int variable, int value) const {
    assert(g_use_redblack_search && g_painting->is_red_var(variable));
    return g_state_packer->get_bit(buffer, variable, value);
}

void State::dump_pddl() const {
    for (size_t var = 0; var < g_variable_domain.size(); ++var) {
        if (g_use_redblack_search && g_painting->is_red_var(var)){
            for (int val = 0; val < g_variable_domain[var]; ++val){
                if (this->has_fact(var, val)){
                    cout << g_fact_names[var][val] << ", ";
                }
            }
            cout << endl;
        } else {
            const string &fact_name = g_fact_names[var][(*this)[var]];
            if (fact_name != "<none of those>")
                cout << fact_name << endl;
        }
    }
}

void State::dump_fdr() const {
    for (size_t i = 0; i < g_variable_domain.size(); ++i){
        cout << "  #" << i << " [" << g_variable_name[i] << "] -> " << (*this)[i] << endl;
    }
}
